
# q-expansion of newform 151.2.a.b, downloaded from the LMFDB on 10 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 151
weight = 2
poly_data = [3, -4, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[0, 1, 0], 1], [[-3, 0, 1], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0, 1], [2, 0, 0], [2, -1, -1], [-2, 0, 0], [-1, 2, -1], [0, -2, 2], [3, 0, -1], [0, 3, 0], [0, 0, 2], [0, 3, 2], [0, -1, 1], [1, 0, -3], [0, 0, 0], [0, -1, 1], [-4, -1, 0], [0, -6, 0], [8, -1, 1], [-4, 4, -4], [2, -4, 4], [0, 0, -2], [4, -2, 2], [-8, -2, 2], [10, -2, 4], [12, 0, 2], [-4, 7, 2], [0, 6, -2], [-5, -6, 3], [2, -4, -2], [2, 0, 0], [0, 6, 4], [-4, 3, -3], [-10, -4, -2], [10, 1, -3], [-8, 5, -8], [10, -2, -2], [1, 0, 0], [8, -2, -4], [-4, 4, -10], [-4, -4, -4], [-8, 7, 6], [4, 4, 2], [-2, 0, -6], [4, 1, 4], [-6, -1, -5], [-2, 4, 0], [-4, 4, 8], [-6, -6, 0], [17, 2, 1], [1, -2, 1], [-4, 5, 10], [4, -2, 0], [-9, 6, -5], [5, 0, 9], [8, -4, 0], [-2, 10, 0], [2, -4, -6], [17, -4, -3], [-14, 8, -2], [-2, 0, -6], [12, -6, -8], [-2, 4, 8], [-10, -4, -2], [-5, 2, -5], [8, -7, 7], [-6, 11, -5], [0, -6, -2], [21, -6, -3], [-10, 0, -6], [-4, -4, -4], [11, 0, 3], [4, -8, 4], [-10, 8, 4], [24, 6, 0], [-4, 6, -6], [-8, -4, -2], [12, -9, -11], [0, 6, 6], [-12, 1, -10], [20, 5, -6], [-6, 12, 0], [-12, 0, -2], [-6, 2, 10], [0, -6, 4], [-12, -6, 6], [9, -2, -7], [0, 6, -2], [0, 0, 2], [22, -1, 7], [-7, 8, 1], [7, 2, 7], [-10, 2, -4], [-12, 0, 6], [8, -3, 12], [24, -3, -1], [-14, -18, 6], [0, 3, -3], [6, -12, 10], [20, -7, -2], [14, -10, -2], [-10, 7, -1], [24, 1, -4], [-4, -4, 12], [-4, -1, -8], [-5, 4, 15], [-4, 7, 5], [8, -5, 2], [-18, 0, -4], [0, -18, -2], [-8, 10, -2], [-2, 1, 17], [0, -6, -6], [-26, 4, 8], [12, -6, -8], [-14, 2, -8], [8, 2, 4], [14, -7, 9], [-16, 19, 8], [-19, 14, 9], [-18, 0, -16], [-16, -7, 3], [10, -2, -10], [17, -12, 9], [-10, -10, 0], [6, -6, -6], [0, -14, 2], [22, 1, 9], [5, -12, -3], [-4, 2, -10], [-12, 0, 0], [6, 8, -14], [-14, -4, -14], [7, -2, 15], [10, -10, -2], [-6, -17, -1], [24, -6, 2], [4, -6, 12], [18, 12, -4], [13, -6, -15], [-16, -1, 14], [4, -2, 12], [-26, 2, 10], [-18, -6, -10], [28, -11, -4], [8, -16, 4], [10, -16, 16], [-4, 11, -8], [14, 5, 13], [-24, -12, 4], [8, 4, 2], [12, 12, 8], [6, -14, 2], [2, 2, -18], [9, -14, -7], [12, 0, -6], [16, 1, 20], [-28, -7, 8], [-6, 4, -10], [2, 8, -12], [16, -19, -2], [-6, 0, -8], [4, 10, 8], [-12, -9, -2], [16, 14, -2], [-4, -22, 6], [-8, 10, 18], [6, 12, -16], [24, 23, -8], [-17, -8, 11]]
